@@ -13,22 +13,16 @@
 use crate::target::Target;
 use crate::TranspilerError;
 use hashbrown::HashSet;
-use pyo3::prelude::*;
-use pyo3::types::PyTuple;
+use pyo3::{intern, prelude::*};
 use qiskit_circuit::bit::{QuantumRegister, Register};
+use qiskit_circuit::converters::dag_to_circuit;
+use qiskit_circuit::imports::QUANTUM_CIRCUIT;
 use qiskit_circuit::operations::OperationRef;
 use qiskit_circuit::packed_instruction::PackedOperation;
 use qiskit_circuit::PhysicalQubit;
 use qiskit_circuit::{
-    converters::{circuit_to_dag, QuantumCircuitData},
-    dag_circuit::DAGCircuit,
-    imports,
-    imports::get_std_gate_class,
-    operations::Operation,
-    operations::Param,
-    operations::StandardGate,
-    packed_instruction::PackedInstruction,
-    Qubit,
+    dag_circuit::DAGCircuit, operations::Operation, operations::Param, operations::StandardGate,
+    packed_instruction::PackedInstruction, Qubit,
 };
 use rustworkx_core::petgraph::stable_graph::NodeIndex;
 use smallvec::{smallvec, SmallVec};
@@ -50,14 +44,13 @@ use std::f64::consts::PI;
 #[pyfunction]
 #[pyo3(name = "check_gate_direction_coupling")]
 pub fn check_direction_coupling_map(
-    py: Python,
     dag: &DAGCircuit,
     coupling_edges: HashSet<[Qubit; 2]>,
 ) -> PyResult<bool> {
     let coupling_map_check =
         |_: &PackedInstruction, op_args: &[Qubit]| -> bool { coupling_edges.contains(op_args) };
 
-    check_gate_direction(py, dag, &coupling_map_check, None)
+    check_gate_direction(dag, &coupling_map_check, None)
 }
 
 /// Check if the two-qubit gates follow the right direction with respect to instructions supported in the given target.
@@ -71,7 +64,7 @@ pub fn check_direction_coupling_map(
 ///     true iff all two-qubit gates comply with the target's coupling constraints
 #[pyfunction]
 #[pyo3(name = "check_gate_direction_target")]
-pub fn check_direction_target(py: Python, dag: &DAGCircuit, target: &Target) -> PyResult<bool> {
+pub fn check_direction_target(dag: &DAGCircuit, target: &Target) -> PyResult<bool> {
     let target_check = |inst: &PackedInstruction, op_args: &[Qubit]| -> bool {
         let qargs = [
             PhysicalQubit::new(op_args[0].0),
@@ -81,7 +74,7 @@ pub fn check_direction_target(py: Python, dag: &DAGCircuit, target: &Target) ->
         target.instruction_supported(inst.op.name(), &qargs)
     };
 
-    check_gate_direction(py, dag, &target_check, None)
+    check_gate_direction(dag, &target_check, None)
 }
 
 // The main routine for checking gate directionality.
@@ -94,7 +87,6 @@ pub fn check_direction_target(py: Python, dag: &DAGCircuit, target: &Target) ->
 //  to carry the mapping context relative to the original DAG.
 //  When qubit_mapping is None, the identity mapping is assumed
 fn check_gate_direction<T>(
-    py: Python,
     dag: &DAGCircuit,
     gate_complies: &T,
     qubit_mapping: Option<&[Qubit]>,
@@ -107,11 +99,9 @@ where
 
         if let OperationRef::Instruction(py_inst) = packed_inst.op.view() {
             if py_inst.control_flow() {
-                let circuit_to_dag = imports::CIRCUIT_TO_DAG.get_bound(py);
-                let py_inst = py_inst.instruction.bind(py);
-
-                for block in py_inst.getattr("blocks")?.try_iter()? {
-                    let inner_dag: DAGCircuit = circuit_to_dag.call1((block?,))?.extract()?;
+                for block in py_inst.blocks() {
+                    let inner_dag =
+                        DAGCircuit::from_circuit_data(&block, false, None, None, None, None)?;
 
                     let block_ok = if let Some(mapping) = qubit_mapping {
                         let mapping = inst_qargs // Create a temp mapping for the recursive call
@@ -119,9 +109,9 @@ where
                             .map(|q| mapping[q.index()])
                             .collect::<Vec<Qubit>>();
 
-                        check_gate_direction(py, &inner_dag, gate_complies, Some(&mapping))?
+                        check_gate_direction(&inner_dag, gate_complies, Some(&mapping))?
                     } else {
-                        check_gate_direction(py, &inner_dag, gate_complies, Some(inst_qargs))?
+                        check_gate_direction(&inner_dag, gate_complies, Some(inst_qargs))?
                     };
 
                     if !block_ok {
@@ -168,18 +158,17 @@ where
 #[pyfunction]
 #[pyo3(name = "fix_gate_direction_coupling")]
 pub fn fix_direction_coupling_map(
-    py: Python,
     dag: &mut DAGCircuit,
     coupling_edges: HashSet<[Qubit; 2]>,
-) -> PyResult<DAGCircuit> {
+) -> PyResult<()> {
     if coupling_edges.is_empty() {
-        return Ok(dag.clone());
+        return Ok(());
     }
 
     let coupling_map_check =
         |_: &PackedInstruction, op_args: &[Qubit]| -> bool { coupling_edges.contains(op_args) };
 
-    fix_gate_direction(py, dag, &coupling_map_check, None).cloned()
+    fix_gate_direction(dag, &coupling_map_check, None)
 }
 
 /// Try to swap two-qubit gate directions using pre-defined mapping to follow the right direction with respect to the given target.
@@ -193,11 +182,7 @@ pub fn fix_direction_coupling_map(
 ///     the transformed DAGCircuit
 #[pyfunction]
 #[pyo3(name = "fix_gate_direction_target")]
-pub fn fix_direction_target(
-    py: Python,
-    dag: &mut DAGCircuit,
-    target: &Target,
-) -> PyResult<DAGCircuit> {
+pub fn fix_direction_target(dag: &mut DAGCircuit, target: &Target) -> PyResult<()> {
     let target_check = |inst: &PackedInstruction, op_args: &[Qubit]| -> bool {
         let qargs = smallvec![
             PhysicalQubit::new(op_args[0].0),
@@ -210,17 +195,13 @@ pub fn fix_direction_target(
                 StandardGate::RXX | StandardGate::RYY | StandardGate::RZZ | StandardGate::RZX => {
                     return target
                         .py_instruction_supported(
-                            py,
-                            None,
+                            Some(std_gate.get_name().to_string()),
                             qargs.into(),
-                            Some(
-                                get_std_gate_class(py, std_gate)
-                                    .expect("These gates should have Python classes")
-                                    .bind(py),
-                            ),
+                            None,
                             Some(inst.params_view().to_vec()),
+                            false,
                         )
-                        .unwrap_or(false)
+                        .unwrap_or(false);
                 }
                 _ => {}
             }
@@ -228,54 +209,45 @@ pub fn fix_direction_target(
         target.instruction_supported(inst.op.name(), &qargs)
     };
 
-    fix_gate_direction(py, dag, &target_check, None).cloned()
+    fix_gate_direction(dag, &target_check, None)
 }
 
-// The main routine for fixing gate direction. Same parameters are check_gate_direction
-fn fix_gate_direction<'a, T>(
-    py: Python,
-    dag: &'a mut DAGCircuit,
+// The main routine for fixing gate direction. Same parameters as check_gate_direction
+fn fix_gate_direction<T>(
+    dag: &mut DAGCircuit,
     gate_complies: &T,
     qubit_mapping: Option<&[Qubit]>,
-) -> PyResult<&'a DAGCircuit>
+) -> PyResult<()>
 where
     T: Fn(&PackedInstruction, &[Qubit]) -> bool,
 {
     let mut nodes_to_replace: Vec<(NodeIndex, DAGCircuit)> = Vec::new();
-    let mut ops_to_replace: Vec<(NodeIndex, Vec<Bound<PyAny>>)> = Vec::new();
+    let mut ops_to_replace = Vec::new();
 
     for (node, packed_inst) in dag.op_nodes(false) {
         let op_args = dag.get_qargs(packed_inst.qubits);
 
         if let OperationRef::Instruction(py_inst) = packed_inst.op.view() {
             if py_inst.control_flow() {
-                let dag_to_circuit = imports::DAG_TO_CIRCUIT.get_bound(py);
-
-                let blocks = py_inst.instruction.bind(py).getattr("blocks")?;
-                let blocks = blocks.downcast::<PyTuple>()?;
+                let blocks = py_inst.blocks();
 
                 let mut blocks_to_replace = Vec::with_capacity(blocks.len());
                 for block in blocks {
-                    let mut inner_dag = circuit_to_dag(
-                        py,
-                        QuantumCircuitData::extract_bound(&block)?,
-                        false,
-                        None,
-                        None,
-                    )?;
+                    let mut inner_dag =
+                        DAGCircuit::from_circuit_data(&block, false, None, None, None, None)?;
 
-                    let inner_dag = if let Some(mapping) = qubit_mapping {
+                    if let Some(mapping) = qubit_mapping {
                         let mapping = op_args // Create a temp mapping for the recursive call
                             .iter()
                             .map(|q| mapping[q.index()])
                             .collect::<Vec<Qubit>>();
 
-                        fix_gate_direction(py, &mut inner_dag, gate_complies, Some(&mapping))?
+                        fix_gate_direction(&mut inner_dag, gate_complies, Some(&mapping))?;
                     } else {
-                        fix_gate_direction(py, &mut inner_dag, gate_complies, Some(op_args))?
+                        fix_gate_direction(&mut inner_dag, gate_complies, Some(op_args))?;
                     };
 
-                    let circuit = dag_to_circuit.call1((inner_dag.clone(),))?;
+                    let circuit = dag_to_circuit(&inner_dag, false)?;
                     blocks_to_replace.push(circuit);
                 }
 
@@ -342,30 +314,40 @@ where
         }
     }
 
-    for (node, op_blocks) in ops_to_replace {
-        let packed_inst = dag[node].unwrap_operation();
-        let OperationRef::Instruction(py_inst) = packed_inst.op.view() else {
-            panic!("PyInstruction is expected");
-        };
-        let new_op = py_inst
-            .instruction
-            .bind(py)
-            .call_method1("replace_blocks", (op_blocks,))?;
+    if !ops_to_replace.is_empty() {
+        Python::with_gil(|py| -> PyResult<()> {
+            for (node, op_blocks) in ops_to_replace {
+                let packed_inst = dag[node].unwrap_operation();
+                let OperationRef::Instruction(py_inst) = packed_inst.op.view() else {
+                    panic!("PyInstruction is expected");
+                };
+                let quantum_circuits = op_blocks
+                    .into_iter()
+                    .map(|block| {
+                        QUANTUM_CIRCUIT
+                            .get_bound(py)
+                            .call_method1(intern!(py, "_from_circuit_data"), (block,))
+                            .expect("Cannot construct QuantumCircuit from CircuitData")
+                    })
+                    .collect::<Vec<Bound<PyAny>>>();
 
-        dag.py_substitute_node(py, dag.get_node(py, node)?.bind(py), &new_op, false, None)?;
+                let new_op = py_inst
+                    .instruction
+                    .bind(py)
+                    .call_method1("replace_blocks", (quantum_circuits,))?;
+
+                dag.py_substitute_node(py, dag.get_node(py, node)?.bind(py), &new_op, false, None)?;
+            }
+
+            Ok(())
+        })?;
     }
 
-    for (node, replacemanet_dag) in nodes_to_replace {
-        dag.py_substitute_node_with_dag(
-            py,
-            dag.get_node(py, node)?.bind(py),
-            &replacemanet_dag,
-            None,
-            None,
-        )?;
+    for (node, replacement_dag) in nodes_to_replace {
+        dag.substitute_node_with_dag(node, &replacement_dag, None, None, None)?;
     }
 
-    Ok(dag)
+    Ok(())
 }
 
 // Return a replacement DAG for the given standard gate in the supported list
@@ -429,120 +411,125 @@ fn apply_operation_back(
 }
 
 fn cx_replacement_dag() -> PyResult<DAGCircuit> {
-    let new_dag = &mut DAGCircuit::new()?;
-    let qargs = add_qreg(new_dag, 2)?;
+    let mut new_dag = DAGCircuit::new();
+    let qargs = add_qreg(&mut new_dag, 2)?;
     let qargs = qargs.as_slice();
 
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[1]], None)?;
-    apply_operation_back(new_dag, StandardGate::CX, &[qargs[1], qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[1]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[1]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::CX, &[qargs[1], qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[1]], None)?;
 
-    Ok(new_dag.clone())
+    Ok(new_dag)
 }
 
 fn ecr_replacement_dag() -> PyResult<DAGCircuit> {
-    let new_dag = &mut DAGCircuit::new()?;
+    let mut new_dag = DAGCircuit::new();
     new_dag.add_global_phase(&Param::Float(-PI / 2.0))?;
-    let qargs = add_qreg(new_dag, 2)?;
+    let qargs = add_qreg(&mut new_dag, 2)?;
     let qargs = qargs.as_slice();
 
-    apply_operation_back(new_dag, StandardGate::S, &[qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::SX, &[qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::Sdg, &[qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::Sdg, &[qargs[1]], None)?;
-    apply_operation_back(new_dag, StandardGate::SX, &[qargs[1]], None)?;
-    apply_operation_back(new_dag, StandardGate::S, &[qargs[1]], None)?;
-    apply_operation_back(new_dag, StandardGate::ECR, &[qargs[1], qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[1]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::S, &[qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::SX, &[qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::Sdg, &[qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::Sdg, &[qargs[1]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::SX, &[qargs[1]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::S, &[qargs[1]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::ECR, &[qargs[1], qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[1]], None)?;
 
-    Ok(new_dag.clone())
+    Ok(new_dag)
 }
 
 fn cz_replacement_dag() -> PyResult<DAGCircuit> {
-    let new_dag = &mut DAGCircuit::new()?;
-    let qargs = add_qreg(new_dag, 2)?;
+    let mut new_dag = DAGCircuit::new();
+    let qargs = add_qreg(&mut new_dag, 2)?;
     let qargs = qargs.as_slice();
 
-    apply_operation_back(new_dag, StandardGate::CZ, &[qargs[1], qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::CZ, &[qargs[1], qargs[0]], None)?;
 
-    Ok(new_dag.clone())
+    Ok(new_dag)
 }
 
 fn swap_replacement_dag() -> PyResult<DAGCircuit> {
-    let new_dag = &mut DAGCircuit::new()?;
-    let qargs = add_qreg(new_dag, 2)?;
-    let qargs = qargs.as_slice();
-
-    apply_operation_back(new_dag, StandardGate::Swap, &[qargs[1], qargs[0]], None)?;
-
-    Ok(new_dag.clone())
-}
-
-fn rxx_replacement_dag(param: &[Param]) -> PyResult<DAGCircuit> {
-    let new_dag = &mut DAGCircuit::new()?;
-    let qargs = add_qreg(new_dag, 2)?;
+    let mut new_dag = DAGCircuit::new();
+    let qargs = add_qreg(&mut new_dag, 2)?;
     let qargs = qargs.as_slice();
 
     apply_operation_back(
-        new_dag,
+        &mut new_dag,
+        StandardGate::Swap,
+        &[qargs[1], qargs[0]],
+        None,
+    )?;
+
+    Ok(new_dag)
+}
+
+fn rxx_replacement_dag(param: &[Param]) -> PyResult<DAGCircuit> {
+    let mut new_dag = DAGCircuit::new();
+    let qargs = add_qreg(&mut new_dag, 2)?;
+    let qargs = qargs.as_slice();
+
+    apply_operation_back(
+        &mut new_dag,
         StandardGate::RXX,
         &[qargs[1], qargs[0]],
         Some(SmallVec::from(param)),
     )?;
 
-    Ok(new_dag.clone())
+    Ok(new_dag)
 }
 
 fn ryy_replacement_dag(param: &[Param]) -> PyResult<DAGCircuit> {
-    let new_dag = &mut DAGCircuit::new()?;
-    let qargs = add_qreg(new_dag, 2)?;
+    let mut new_dag = DAGCircuit::new();
+    let qargs = add_qreg(&mut new_dag, 2)?;
     let qargs = qargs.as_slice();
 
     apply_operation_back(
-        new_dag,
+        &mut new_dag,
         StandardGate::RYY,
         &[qargs[1], qargs[0]],
         Some(SmallVec::from(param)),
     )?;
 
-    Ok(new_dag.clone())
+    Ok(new_dag)
 }
 
 fn rzz_replacement_dag(param: &[Param]) -> PyResult<DAGCircuit> {
-    let new_dag = &mut DAGCircuit::new()?;
-    let qargs = add_qreg(new_dag, 2)?;
+    let mut new_dag = DAGCircuit::new();
+    let qargs = add_qreg(&mut new_dag, 2)?;
     let qargs = qargs.as_slice();
 
     apply_operation_back(
-        new_dag,
+        &mut new_dag,
         StandardGate::RZZ,
         &[qargs[1], qargs[0]],
         Some(SmallVec::from(param)),
     )?;
 
-    Ok(new_dag.clone())
+    Ok(new_dag)
 }
 
 fn rzx_replacement_dag(param: &[Param]) -> PyResult<DAGCircuit> {
-    let new_dag = &mut DAGCircuit::new()?;
-    let qargs = add_qreg(new_dag, 2)?;
+    let mut new_dag = DAGCircuit::new();
+    let qargs = add_qreg(&mut new_dag, 2)?;
     let qargs = qargs.as_slice();
 
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[1]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[1]], None)?;
     apply_operation_back(
-        new_dag,
+        &mut new_dag,
         StandardGate::RZX,
         &[qargs[1], qargs[0]],
         Some(SmallVec::from(param)),
     )?;
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[0]], None)?;
-    apply_operation_back(new_dag, StandardGate::H, &[qargs[1]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[0]], None)?;
+    apply_operation_back(&mut new_dag, StandardGate::H, &[qargs[1]], None)?;
 
-    Ok(new_dag.clone())
+    Ok(new_dag)
 }
 
 pub fn gate_direction_mod(m: &Bound<PyModule>) -> PyResult<()> {
