@@ -29,7 +29,7 @@ from ..n_local.n_local import NLocal
 
 
 def _normalize_entanglement(
-    entanglement: str | Mapping[int, Sequence[Sequence[int]]]
+    entanglement: str | Mapping[int, Sequence[Sequence[int]]],
 ) -> str | dict[int, list[tuple[int]]]:
     if isinstance(entanglement, str):
         return entanglement
@@ -199,6 +199,7 @@ def z_feature_map(
 
     Examples:
 
+        >>> from qiskit.circuit.library import z_feature_map
         >>> prep = z_feature_map(3, reps=3, insert_barriers=True)
         >>> print(prep)
              ┌───┐ ░ ┌─────────────┐ ░ ┌───┐ ░ ┌─────────────┐ ░ ┌───┐ ░ ┌─────────────┐
@@ -220,9 +221,10 @@ def z_feature_map(
         q_2: ┤ H ├┤ P(2.0*x[2]**2 + 2.0) ├
              └───┘└──────────────────────┘
 
-        >>> from qiskit.circuit.library import TwoLocal
-        >>> ry = TwoLocal(3, "ry", "cz", reps=1).decompose()
-        >>> classifier = z_feature_map(3, reps=1) + ry
+        >>> from qiskit.circuit.library import n_local
+        >>> circuit = n_local(3, "ry", "cz", reps=1).decompose()
+        >>> classifier = z_feature_map(3, reps=1)
+        >>> classifier.append(circuit, list(range(classifier.num_qubits)))
         >>> print(classifier)
              ┌───┐┌─────────────┐┌──────────┐      ┌──────────┐
         q_0: ┤ H ├┤ P(2.0*x[0]) ├┤ RY(θ[0]) ├─■──■─┤ RY(θ[3]) ├────────────
@@ -277,7 +279,7 @@ def zz_feature_map(
 
     Examples:
 
-        >>> from qiskit.circuit.library import ZZFeatureMap
+        >>> from qiskit.circuit.library import zz_feature_map
         >>> prep = zz_feature_map(2, reps=1)
         >>> print(prep)
              ┌───┐┌─────────────┐
@@ -289,7 +291,7 @@ def zz_feature_map(
         >>> from qiskit.circuit.library import efficient_su2
         >>> classifier = zz_feature_map(3).compose(efficient_su2(3))
         >>> classifier.num_parameters
-        15
+        27
         >>> classifier.parameters  # 'x' for the data preparation, 'θ' for the SU2 parameters
         ParameterView([
             ParameterVectorElement(x[0]), ParameterVectorElement(x[1]),
@@ -431,8 +433,7 @@ class PauliFeatureMap(NLocal):
         insert_barriers: bool = False,
         name: str = "PauliFeatureMap",
     ) -> None:
-        """Create a new Pauli expansion circuit.
-
+        """
         Args:
             feature_dimension: Number of qubits in the circuit.
             reps: The number of repeated circuits.
@@ -450,7 +451,7 @@ class PauliFeatureMap(NLocal):
                 default mapping from :meth:`self_product`.
             parameter_prefix: The prefix used if default parameters are generated.
             insert_barriers: If True, barriers are inserted in between the evolution instructions
-                and hadamard layers.
+                and Hadamard layers.
 
         """
 
