@@ -15,28 +15,26 @@
 from qiskit.circuit.quantumcircuit import QuantumCircuit
 
 
-def clifford_8_2():
+def clifford_6_5():
     """
-    Clifford template 8_2:
+    Clifford template 6_5:
 
     .. code-block:: text
 
-                             ┌───┐
-        q_0: ──■─────────■───┤ S ├───■────────────
-             ┌─┴─┐┌───┐┌─┴─┐┌┴───┴┐┌─┴─┐┌───┐┌───┐
-        q_1: ┤ X ├┤ H ├┤ X ├┤ SDG ├┤ X ├┤ S ├┤ H ├
-             └───┘└───┘└───┘└─────┘└───┘└───┘└───┘
+                      ┌───┐
+        q_0: ─■───■───┤ S ├───■───────
+              │ ┌─┴─┐┌┴───┴┐┌─┴─┐┌───┐
+        q_1: ─■─┤ X ├┤ SDG ├┤ X ├┤ S ├
+                └───┘└─────┘└───┘└───┘
 
     Returns:
         QuantumCircuit: template as a quantum circuit.
     """
     qc = QuantumCircuit(2)
-    qc.cx(0, 1)
-    qc.h(1)
+    qc.cz(0, 1)
     qc.cx(0, 1)
     qc.s(0)
     qc.sdg(1)
     qc.cx(0, 1)
     qc.s(1)
-    qc.h(1)
     return qc
