@@ -13,6 +13,12 @@
 """Synthesis for arithmetic circuits."""
 
 from .comparators import synth_integer_comparator_2s, synth_integer_comparator_greedy
-from .adders import adder_qft_d00, adder_ripple_c04, adder_ripple_v95, adder_ripple_r25
+from .adders import (
+    adder_qft_d00,
+    adder_ripple_c04,
+    adder_ripple_v95,
+    adder_ripple_r25,
+    adder_modular_v17,
+)
 from .multipliers import multiplier_cumulative_h18, multiplier_qft_r17
 from .weighted_sum import synth_weighted_sum_carry
